@@ -1,7 +1,11 @@
 use anyhow::anyhow;
 use etcetera::base_strategy::{choose_base_strategy, BaseStrategy};
 use serde::{de, Deserialize, Deserializer};
-use std::{fs, path::PathBuf, sync::OnceLock};
+use std::{
+    fs,
+    path::{Path, PathBuf},
+    sync::OnceLock,
+};
 use syntect::highlighting::{Theme, ThemeSet};
 
 pub const APP_NAME: &str = "scooter";
@@ -18,7 +22,18 @@ fn get_theme_set() -> &'static ThemeSet {
     })
 }
 
+static CONFIG_DIR_OVERRIDE: OnceLock<PathBuf> = OnceLock::new();
+
+pub fn set_config_dir_override(dir: &Path) {
+    CONFIG_DIR_OVERRIDE
+        .set(dir.to_path_buf())
+        .expect("Config dir override should only be set once");
+}
+
 fn config_dir() -> PathBuf {
+    if let Some(dir) = CONFIG_DIR_OVERRIDE.get() {
+        return dir.clone();
+    }
     let strategy = choose_base_strategy().expect("Unable to find config directory!");
     strategy.config_dir().join(APP_NAME)
 }
@@ -86,10 +101,10 @@ pub struct EditorOpenConfig {
     /// [editor_open]
     /// command = "vi %file +%line"
     /// ```
-    /// If not set explicitly, Scooter will attempt to use the editor set by the `$EDITOR` environment variable.
+    /// If not set explicitly, scooter will attempt to use the editor set by the `$EDITOR` environment variable.
     #[serde(default)]
     pub command: Option<String>,
-    /// Whether to exit Scooter after running the command defined by `editor_open.command`. Defaults to `false`.
+    /// Whether to exit scooter after running the command defined by `editor_open.command`. Defaults to `false`.
     #[serde(default = "default_exit")]
     pub exit: bool,
 }
@@ -114,8 +129,10 @@ pub struct PreviewConfig {
     /// The default is `"base16-eighties.dark"`. Other built-in options are
     /// `"base16-mocha.dark"`, `"base16-ocean.dark"`, `"base16-ocean.light"`, `"InspiredGitHub"`, `"Solarized (dark)"` and `"Solarized (light)"`.
     ///
-    /// You can use other themes by adding `.tmTheme` files to `~/.config/scooter/themes/` on Linux or macOS, or `%AppData%\scooter\themes\` on Windows,
-    /// and then specifying their name here. For instance, to use Catppuccin Macchiato (from [here](https://github.com/catppuccin/bat)), on Linux or macOS run:
+    /// You can use other themes by adding `.tmTheme` files to `<scooter-config-dir>/themes` and then specifying their name here.
+    /// By default, `<scooter-config-dir>` is `~/.config/scooter/` on Linux or macOS, or `%AppData%\scooter\` on Windows, and can be overridden with the `--config-dir` flag.
+    ///
+    /// For instance, to use Catppuccin Macchiato (from [here](https://github.com/catppuccin/bat)), on Linux or macOS run:
     /// ```sh
     /// wget -P ~/.config/scooter/themes https://github.com/catppuccin/bat/raw/main/themes/Catppuccin%20Macchiato.tmTheme
     /// ```
@@ -167,7 +184,7 @@ where
 #[serde(deny_unknown_fields)]
 pub struct StyleConfig {
     /// Force enable or disable true color. `true` forces true color (supported by most modern terminals but not e.g. Apple Terminal), while `false` forces 256 colors (supported by almost all terminals including Apple Terminal).
-    /// If omitted, Scooter will attempt to determine whether the terminal being used supports true color.
+    /// If omitted, scooter will attempt to determine whether the terminal being used supports true color.
     #[serde(default = "detect_true_colour")]
     pub true_color: bool,
 }
