@@ -1,24 +1,29 @@
 use anyhow::bail;
-use app::AppRunConfig;
 use clap::Parser;
+use frep_core::validation::{DirConfig, SearchConfig};
 use log::LevelFilter;
-use std::{path::PathBuf, str::FromStr};
+use std::{
+    io::{self, IsTerminal, Read},
+    path::PathBuf,
+    str::FromStr,
+};
+
+use scooter_core::{
+    app::AppRunConfig,
+    fields::{FieldValue, SearchFieldValues},
+};
 
 use app_runner::{run_app_tui, AppConfig};
-use fields::{FieldValue, SearchFieldValues};
-use headless::run_headless;
+use headless::{run_headless, run_headless_with_stdin};
 use logging::{setup_logging, DEFAULT_LOG_LEVEL};
 
-mod app;
 mod app_runner;
 mod config;
-mod fields;
+mod conversions;
 mod headless;
 mod logging;
-mod replace;
 mod tui;
 mod ui;
-mod utils;
 
 #[derive(Parser, Debug)]
 #[command(about = "Interactive find and replace TUI.")]
@@ -26,7 +31,7 @@ mod utils;
 #[allow(clippy::struct_excessive_bools)]
 struct Args {
     /// Directory in which to search
-    #[arg(index = 1, value_parser = parse_directory, default_value = ".")]
+    #[arg(index = 1, value_parser = parse_search_dir, default_value = ".")]
     directory: PathBuf,
 
     /// Include hidden files and directories, such as those whose name starts with a dot (.)
@@ -61,10 +66,14 @@ struct Args {
     #[arg(short = 'X', long)]
     immediate: bool,
 
-    /// Run Scooter without a TUI. Search and replace runs immediately (as with the `--immediate` flag), but with no user interface
+    /// Run scooter without a TUI. Search and replace runs immediately (as with the `--immediate` flag), but with no user interface
     #[arg(short = 'N', long)]
     no_tui: bool,
 
+    /// Override the config directory (default: ~/.config/scooter on Linux/macOS, %AppData%\scooter on Windows)
+    #[arg(short = 'c', long, value_parser = parse_config_dir)]
+    config_dir: Option<PathBuf>,
+
     // --- Initial values for fields ---
     //
     /// Text to search with
@@ -100,7 +109,7 @@ fn parse_log_level(s: &str) -> Result<LevelFilter, String> {
     LevelFilter::from_str(s).map_err(|_| format!("Invalid log level: {s}"))
 }
 
-fn parse_directory(dir: &str) -> anyhow::Result<PathBuf> {
+fn parse_search_dir(dir: &str) -> anyhow::Result<PathBuf> {
     let path = PathBuf::from(dir);
     if path.exists() {
         Ok(path)
@@ -109,6 +118,14 @@ fn parse_directory(dir: &str) -> anyhow::Result<PathBuf> {
     }
 }
 
+fn parse_config_dir(dir: &str) -> anyhow::Result<PathBuf> {
+    let path = PathBuf::from(dir);
+    if path.exists() && !path.is_dir() {
+        bail!("'{dir}' exists but is not a directory. Please provide a valid directory path.")
+    }
+    Ok(path)
+}
+
 fn validate_flag_combinations(args: &Args) -> anyhow::Result<()> {
     if args.no_tui && args.immediate {
         bail!("--no-tui cannot be combined with --immediate");
@@ -126,11 +143,7 @@ fn validate_flag_combinations(args: &Args) -> anyhow::Result<()> {
 }
 
 fn validate_search_text_required(args: &Args) -> anyhow::Result<()> {
-    if args
-        .search_text
-        .as_ref()
-        .is_none_or(std::string::String::is_empty)
-    {
+    if args.search_text.as_ref().is_none_or(String::is_empty) {
         for (name, enabled) in [
             ("--immediate-search", args.immediate_search),
             ("--immediate", args.immediate),
@@ -145,12 +158,42 @@ fn validate_search_text_required(args: &Args) -> anyhow::Result<()> {
     Ok(())
 }
 
+fn detect_and_read_stdin() -> anyhow::Result<Option<String>> {
+    if io::stdin().is_terminal() {
+        return Ok(None);
+    }
+
+    let mut stdin_content = String::new();
+    io::stdin().read_to_string(&mut stdin_content)?;
+
+    Ok(Some(stdin_content))
+}
+
+fn validate_stdin_usage(args: &Args, stdin_content: Option<&str>) -> anyhow::Result<()> {
+    if stdin_content.is_some() {
+        // File system args
+        if args.hidden {
+            bail!("Cannot use --hidden flag when processing stdin");
+        }
+        if args.files_to_include.is_some() {
+            bail!("Cannot use --files-to-include when processing stdin");
+        }
+        if args.files_to_exclude.is_some() {
+            bail!("Cannot use --files-to-exclude when processing stdin");
+        }
+    }
+    Ok(())
+}
+
 impl<'a> TryFrom<&'a Args> for AppConfig<'a> {
     type Error = anyhow::Error;
 
     fn try_from(args: &'a Args) -> anyhow::Result<Self> {
+        let stdin_content = detect_and_read_stdin()?;
+
         validate_flag_combinations(args)?;
         validate_search_text_required(args)?;
+        validate_stdin_usage(args, stdin_content.as_deref())?;
 
         let immediate = args.immediate || args.no_tui;
 
@@ -165,6 +208,7 @@ impl<'a> TryFrom<&'a Args> for AppConfig<'a> {
                 immediate_replace: args.immediate_replace || immediate,
                 print_results: args.print_results || immediate,
             },
+            stdin_content,
         })
     }
 }
@@ -203,22 +247,50 @@ impl<'a> From<&'a Args> for SearchFieldValues<'a> {
 #[tokio::main]
 async fn main() -> anyhow::Result<()> {
     let args = Args::parse();
+    if let Some(config_dir) = &args.config_dir {
+        config::set_config_dir_override(config_dir);
+    }
     let config = AppConfig::try_from(&args)?;
     setup_logging(config.log_level)?;
 
     let results = if args.no_tui {
-        let res = run_headless(config.try_into()?)?;
-        Some(res)
+        let results = if let Some(stdin_content) = config.stdin_content {
+            run_headless_with_stdin(&stdin_content, search_config_from_args(&args))?
+        } else {
+            run_headless(search_config_from_args(&args), dir_config_from_args(&args))?
+        };
+        Some(results)
     } else {
         run_app_tui(config).await?
     };
 
     if let Some(results) = results {
-        println!("{results}");
+        print!("{results}");
     }
+
     Ok(())
 }
 
+fn dir_config_from_args(args: &Args) -> DirConfig<'_> {
+    DirConfig {
+        include_globs: args.files_to_include.as_deref(),
+        exclude_globs: args.files_to_exclude.as_deref(),
+        include_hidden: args.hidden,
+        directory: args.directory.clone(),
+    }
+}
+
+fn search_config_from_args(args: &Args) -> SearchConfig<'_> {
+    SearchConfig {
+        search_text: args.search_text.as_deref().unwrap_or(""),
+        replacement_text: args.replace_text.as_deref().unwrap_or(""),
+        fixed_strings: args.fixed_strings,
+        advanced_regex: args.advanced_regex,
+        match_whole_word: args.match_whole_word,
+        match_case: !args.case_insensitive,
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -244,6 +316,7 @@ mod tests {
             case_insensitive: false,
             files_to_include: None,
             files_to_exclude: None,
+            config_dir: None,
         }
     }
 
@@ -492,7 +565,7 @@ mod tests {
         let temp_dir = setup_test_dir();
         let dir_path = temp_dir.path().to_str().unwrap();
 
-        let result = parse_directory(dir_path);
+        let result = parse_search_dir(dir_path);
         assert!(result.is_ok());
         assert_eq!(result.unwrap(), PathBuf::from(dir_path));
     }
@@ -500,7 +573,7 @@ mod tests {
     #[test]
     fn test_validate_directory_does_not_exist() {
         let nonexistent_path = "/path/that/definitely/does/not/exist/12345";
-        let result = parse_directory(nonexistent_path);
+        let result = parse_search_dir(nonexistent_path);
 
         assert!(result.is_err());
         let err = result.unwrap_err().to_string();
@@ -515,7 +588,7 @@ mod tests {
         std::fs::create_dir_all(&nested_dir).expect("Failed to create nested directories");
 
         let dir_path = nested_dir.to_str().unwrap();
-        let result = parse_directory(dir_path);
+        let result = parse_search_dir(dir_path);
 
         assert!(result.is_ok());
         assert_eq!(result.unwrap(), nested_dir);
@@ -529,9 +602,40 @@ mod tests {
             .expect("Failed to create directory with special characters");
 
         let dir_path = special_dir.to_str().unwrap();
-        let result = parse_directory(dir_path);
+        let result = parse_search_dir(dir_path);
 
         assert!(result.is_ok());
         assert_eq!(result.unwrap(), special_dir);
     }
+
+    #[test]
+    fn test_parse_config_dir_valid_directory() {
+        let temp_dir = setup_test_dir();
+        let dir_path = temp_dir.path().to_str().unwrap();
+
+        let result = parse_config_dir(dir_path);
+        assert!(result.is_ok());
+        assert_eq!(result.unwrap(), temp_dir.path());
+    }
+
+    #[test]
+    fn test_parse_config_dir_nonexistent_path() {
+        // Non-existent paths are allowed
+        let result = parse_config_dir("/path/that/does/not/exist/config");
+        assert!(result.is_ok());
+    }
+
+    #[test]
+    fn test_parse_config_dir_file_not_directory() {
+        let temp_dir = setup_test_dir();
+        let file_path = temp_dir.path().join("test_file.txt");
+        std::fs::write(&file_path, "test").expect("Failed to create test file");
+
+        let file_path_str = file_path.to_str().unwrap();
+        let result = parse_config_dir(file_path_str);
+
+        assert!(result.is_err());
+        let err = result.unwrap_err().to_string();
+        assert!(err.contains("not a directory"));
+    }
 }
